package com.engine.organization.entity.department.bo;

import com.engine.organization.entity.department.dto.DepartmentListDTO;
import com.engine.organization.entity.department.param.DeptSearchParam;
import com.engine.organization.entity.department.po.DepartmentPO;
import com.engine.organization.entity.department.vo.SingleDeptTreeVO;
import com.engine.organization.mapper.department.DepartmentMapper;
import com.engine.organization.mapper.employee.EmployeeMapper;
import com.engine.organization.util.db.MapperProxyFactory;
import org.apache.commons.collections.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author weaver_cl
 * @Description: TODO
 * @Date 2022/5/20
 * @Version V1.0
 **/
public class DepartmentBO {

    public static List<DepartmentListDTO> buildDeptDTOList(Collection<DepartmentPO> list, List<DepartmentPO> filterList) {
        // 搜索结果为空，直接返回空
        if (CollectionUtils.isEmpty(filterList)) {
            return Collections.emptyList();
        }
        // 递归添加父级数据
        Map<Long, DepartmentPO> poMaps = list.stream().collect(Collectors.toMap(item -> item.getId(), item -> item));
        List<DepartmentPO> addedList = new ArrayList<>();
        for (DepartmentPO po : filterList) {
            dealParentData(addedList, po, poMaps);
        }

        List<DepartmentListDTO> dtoList = addedList.stream().map(e -> DepartmentListDTO.builder()
                .id(e.getId())
                .deptNo(e.getDeptNo())
                .deptName(e.getDeptName())
                .deptNameShort(e.getDeptNameShort())
                .parentComp(e.getParentComp() + "")// 命名
                .parentDept(e.getParentDept())
                .parentDeptName(null == poMaps.get(e.getParentDept()) ? "" : poMaps.get(e.getParentDept()).getDeptName())
                .deptPrincipal(getEmployeeNameById((long) e.getDeptPrincipal()))
                .showOrder(e.getShowOrder())
                .forbiddenTag(e.getForbiddenTag())
                .build()
        ).collect(Collectors.toList());
        Map<Long, List<DepartmentListDTO>> collects = dtoList.stream().filter(item -> null != item.getParentDept() && 0 != item.getParentDept()).collect(Collectors.groupingBy(DepartmentListDTO::getParentDept));
        return dtoList.stream().map(e -> {
            e.setChildren(collects.get(e.getId()));
            return e;
        }).filter(item -> null == item.getParentDept() || 0 == item.getParentDept()).collect(Collectors.toList());
    }

    public static DepartmentPO convertParamsToPO(DeptSearchParam param, Long employeeId) {
        if (null == param) {
            return null;
        }
        return DepartmentPO.builder()
                .id(param.getId() == null ? 0 : param.getId())
                .deptNo(param.getDeptNo())
                .deptName(param.getDeptName())
                .deptNameShort(param.getDeptNameShort())
                .parentComp(param.getParentComp())
                .parentDept(param.getParentDept())
                .deptPrincipal(param.getDeptPrincipal())
                .showOrder(param.getShowOrder())
                .forbiddenTag(param.getForbiddenTag() == null ? null : param.getForbiddenTag() ? 0 : 1)
                .deleteType(0)
                .createTime(new Date())
                .updateTime(new Date())
                .creator(employeeId).build();
    }

    public static List<SingleDeptTreeVO> buildSingleDeptTreeVOS(List<DepartmentPO> departmentPOs) {

        if (CollectionUtils.isEmpty(departmentPOs)) {
            return Collections.emptyList();
        }


        List<SingleDeptTreeVO> singleDeptTreeVOS = departmentPOs.stream().map(e -> SingleDeptTreeVO.builder()
                .id(e.getId())
                .deptName(e.getDeptName())
                .parentDeptName(getDeptNameById(e.getParentDept().intValue()))
                .deptPrincipalName(getEmployeeNameById((long) e.getDeptPrincipal()))
                .children(recursiveData(e.getId()))
                .build()
        ).collect(Collectors.toList());

        return singleDeptTreeVOS;

    }


    public static String getDeptNameById(Integer id) {
        return MapperProxyFactory.getProxy(DepartmentMapper.class).getDeptNameById(id);
    }

    public static String getEmployeeNameById(Long id) {
        return MapperProxyFactory.getProxy(EmployeeMapper.class).getEmployeeNameById(id);
    }


    /**
     * 递归获取查询后数据的父级数据
     *
     * @param addedList
     * @param po
     * @param poMaps
     */
    private static void dealParentData(List<DepartmentPO> addedList, DepartmentPO po, Map<Long, DepartmentPO> poMaps) {
        if (!addedList.contains(po)) {
            addedList.add(po);
        }
        DepartmentPO parentPO = poMaps.get(po.getParentDept());
        if (null != parentPO) {
            dealParentData(addedList, parentPO, poMaps);
        }
    }

    /**
     * 递归获取
     *
     * @param parentDeptId
     * @return
     */
    public static List<SingleDeptTreeVO> recursiveData(long parentDeptId) {
        List<DepartmentPO> departmentPOS = MapperProxyFactory.getProxy(DepartmentMapper.class).getDeptListByPId(parentDeptId);
        if (CollectionUtils.isEmpty(departmentPOS)) {
            return Collections.emptyList();
        }

        List<SingleDeptTreeVO> singleDeptTreeVOS = departmentPOS.stream().map(e -> SingleDeptTreeVO.builder()
                .id(e.getId())
                .deptName(e.getDeptName())
                .parentDeptName(getDeptNameById(e.getParentDept().intValue()))
                .deptPrincipalName(getEmployeeNameById((long) e.getDeptPrincipal()))
                .children(recursiveData(e.getId()))
                .build()
        ).collect(Collectors.toList());

        return singleDeptTreeVOS;
    }


}
